import axios from 'axios';
import { cache } from '../server';

async function findMastodonUser(username: string, host: string): Promise<string> {
  const res = await axios.get(`${host}/api/v2/search?q=${username}`, {
    headers: { 'User-Agent': 'EGOport/1.0.0 <matyas@caras.cafe>' },
  });
  if (res.status > 399) {
    console.error(
      `Error ${res.status}, could not find Mastodon profile ID, server returned the following response:`,
    );
    console.error(res.data);
    return '';
  }
  return res.data['accounts'][0]['id'];
}

interface MastodonPost {
  url: string;
  content: string;
  favourites: number;
  reblogs: number;
  replies: number;
  inReplyTo?: string;
  mediaUrls: string[];
  user: string;
}

async function getLatestMastodonPost(id: string, host: string): Promise<MastodonPost> {
  const cached = cache.get('mastodon') as MastodonPost | undefined;
  if (cached != undefined) {
    console.info('Returning mastodon from cache');
    return cached;
  }
  const res = await axios.get(
    `${host}/api/v1/accounts/${id}/statuses?limit=1&exclude_reblogs=true`,
    {
      headers: { 'User-Agent': 'EGOport/1.0.0 <matyas@caras.cafe>' },
    },
  );
  if (res.status > 399) {
    console.error(`Could not get latest posts for id '${id}' using host '${host}':`);
    throw new Error(res.data);
  }
  const data = res.data[0];
  let inReplyTo = undefined;
  if (data['in_reply_to_account_id'] != null) {
    inReplyTo = data['mentions'][0]['acct'];
  }
  const mediaUrl: string[] = [];
  data['media_attachments'].forEach((media: { [key: string]: string | object | null }) => {
    if (media['url'] != null) {
      mediaUrl.push(media['url'] as string);
    }
  });
  const postData = {
    url: data['url'],
    content: decodeURI(data['content']),
    favourites: data['favourites_count'],
    reblogs: data['reblogs_count'],
    replies: data['replies_count'],
    inReplyTo: inReplyTo,
    mediaUrls: mediaUrl,
    user: data['account']['username'],
  };
  cache.set('mastodon', postData);
  console.info('Saved mastodon to cache');
  return postData;
}

export { findMastodonUser, getLatestMastodonPost };
