import fastify from 'fastify';
import config from './plugins/config.js';
import routes from './routes/index.js';
import * as customRoutes from './routes/custom.js';
import fastifyStatic from '@fastify/static';
import fastifyView from '@fastify/view';
import ejs from 'ejs';
import path from 'path';
import { readFileSync } from 'fs';
import { createSchema as S, TsjsonParser } from 'ts-json-validator';
import NodeCache from "node-cache"

const cache = new NodeCache({
  stdTTL: 3600
});

// load userconfig
const parser = new TsjsonParser(
  S({
    title: 'EGOport Configuration',
    description: 'Configuration for the EGOport program',
    type: 'object',
    additionalProperties:S(false),
    properties: {
      theme: S({
        description: "The name of the theme matching a name of a folder inside 'themes'",
        type: 'string',
        default: 'default',
      }),
      title: S({
        description: 'Heading of the page',
        type: 'string',
        default: '',
      }),
      subtitle: S({
        description: 'Normally displayed under the title or somewhere near it',
        type: 'string',
        default: '',
      }),
      links: S({
        type: 'array',
        description: 'An array of links, that can be displayed on your profile',
        items: S({
          type: 'string',
        }),
        default: [],
      }),
      mastodon: S({
        type: 'object',
        description: 'Contains configuration for the Mastodon integration',
        default: {},
        additionalProperties:S(false),
        properties: {
          host: S({
            type: 'string',
            description:
              'A Mastodon host that will be used as the API client, probably should be your home server',
          }),
          username: S({
            type: 'string',
            description: 'Your username on Mastodon',
          }),
        },
      }),
      shoutbox: S({
        type: 'boolean',
        description:
          'Set to true if you want to enable the Shoutbox/Messagewall. Requires a correctly configured theme!',
        default: false,
      }),
      contact: S({
        type: 'boolean',
        description:
          'Set to true if you want to enable the contact form. Requires a correctly configured theme!',
        default: false,
      }),
      captcha: S({
        type:"object",
        additionalProperties:S(false),
        properties:{
          enabled: S({
            type:"boolean",
            description:"Whether captcha should be enabled on shoutbox/contact form",
            default:true
          }),
          method: S({
            type:"string",
            description:"Who will provide the CAPTCHA service",
            enum: ["friendly","mosparo"],
            default: "mosparo"
          }),
          clientKey: S({
            type:"string",
            description:"The site key (Friendly Captcha) or UUID (Mosparo)"
          }),
          secretKey: S({
            type:"string",
            description:"Secret key (Friendly Captcha) or Private key (Mosparo)"
          }),
          url: S({
            type:"string",
            description:"Mosparo instance URL"
          })
        }
      })
    },
    required: ['theme'],
  }),
);
export { parser, cache };

let userConfig;
try {
  userConfig = parser.parse(
    readFileSync(path.join(path.resolve(path.dirname('')), 'config.json'), { encoding: 'utf-8' }),
  );
} catch (error) {
  console.error('Error parsing userconfig! Parser threw the following error:');
  console.error(error);
  console.error('Check your config.json file!');
  process.exit(1);
}

const server = fastify({
  ajv: {
    customOptions: {
      removeAdditional: 'all',
      coerceTypes: true,
      useDefaults: true,
    },
  },
  logger: {
    level: process.env.LOG_LEVEL,
  },
});

await server.register(config);
await server.register(routes);
await server.register(customRoutes.default);
await server.register(fastifyView, {
  engine: {
    ejs: ejs,
  },
  root: path.join(path.resolve(path.dirname('')), 'themes', userConfig.theme),
});
await server.register(fastifyStatic, {
  root: path.join(path.resolve(path.dirname('')), 'themes', userConfig.theme, 'assets'),
  prefix: '/assets/',
});
await server.ready();

export default server;
